<?php

/** 
 * @file axioms.inc
 *
 * Defines the terms, and the terms that define the terms,
 * That make basic statements about what the statements mean :-B
 *
 * OK, sorry, axioms are the things that are so basic to the workings
 * of grammer that you take them for granted.
 * The term 'type' indicates the 'type' of a thing. Simple enough, 
 * but it has to be said before we can ask for all things of a given 'type'.
 *
 * Some basic vocabulary that forms the building blocks for later
 * definitions of statements. They are statements in themselves,
 * but really part of the core workings.
 *
 * Quirky thing to note is how sortOrder is used in the interface.
 * A term with sortOrder (weighting) less than zero does not show
 * up in most user lists, it's regarded as technical.
 * A term with sortOrder > 5 is so important it should even show up in summaries.
 * Thus you can tweak the interface available to editors without really
 * changing the properties of a predicate.
 * Setting the option 'List Advanced Ontologies' however will override this
 * filter and always show all.
 * 
 * Also, much of the settings here control how we co-operate with Drupal
 * internals. The definition of "what is a node" is expressed here in a 
 * standard form. Terms that are given a 'drupalEquivalent' characteristic
 * count as 'implicit' system properties, and require special rights to
 * access.
 *
 * @todo soon enough, all the statements made in this file should be re-parsed
 * into true RDF that I can bootstrap from. The struct array format used here
 * is was just used in the beginning, before I even had a data model.
 * The only question is - how much will requiring XML support on the server
 * limit the audience? (I'm not gonna parse by hand!)
 * Or should I redistribute the ARC thing as core?
 *
 * @ingroup docs
 */
 

/**
 * The namespaces we use to make the term definitions
 * @todo find a tidier way of looking after namespaces
 */
define ('RDF_NS',    'http://www.w3.org/1999/02/22-rdf-syntax-ns#');
define ('RDFS_NS',   'http://www.w3.org/2000/01/rdf-schema#');

define ('OWL_NS',    'http://www.w3.org/2002/07/owl#');
 
define ('DCTERMS_NS','http://purl.org/dc/terms/');
define ('DC_NS',     'http://purl.org/dc/elements/1.1/');

/**
 * The new terms and classes I'm inventing for use within this project will be
 * tied to the project itself. Drupal-specific terminology will use a Drupal-
 * specific URI. 
 * Note that a RDF Resource URI does NOT always have to point to a resource. It
 * would be nice, but the URIs don't even have to resolve. I'd love to point to
 * an RDF schema here instead ...
 */
define ('DRUPAL_NS', 'http://drupal.org/project/relationship#');

/**
 * ROR (Resources about Resources) will be used to encapsulate 
 * some of the extra drupal-specific defined properties
 * ror:sortOrder <==> Drupal:weight
 * We'll continue to use the term 'weight' internally for drupal-ness
 * but rename it and qualify it as ror:sortOrder for serialization
 */ 
define ('ROR_NS',    'http://rorweb.com/0.1/'); 


global $predicate_namespaces;
$predicate_namespaces = array(
  'rdf'     => RDF_NS,
  'rdfs'    => RDFS_NS,
  'owl'     => OWL_NS,
  'dcterms' => DCTERMS_NS,
  'dc'      => DC_NS,
  'ror'     => ROR_NS,
  'drupal'  => DRUPAL_NS,
);

// Classes most often go on the recieving end of 'type' statements.
// so X is a typeOf 'Resource' etc.
 define('CLASS_PROPERTY',            'Property');
 define('CLASS_OBJECT_PROPERTY',     'objectProperty');
 define('CLASS_CLASS',               'Class');
 define('CLASS_RESOURCE',            'Resource');
 define('CLASS_LITERAL',             'Literal');
 define('CLASS_ONTOLOGY',            'Ontology');

 // Drupal - only 
 define('CLASS_NODE',                'Node');
 define('CLASS_USER',                'User');
 define('CLASS_VOCABULARY',          'Vocabulary');


// Predicates are actually known as 'properties' in RDF-speak,
// as some of them have literal objects.
 define('PREDICATE_TITLE',           'title');
 define('PREDICATE_LABEL',           'label');
 define('PREDICATE_IS_DEFINED_BY',   'isDefinedBy');
 define('PREDICATE_SEE_ALSO',        'seeAlso'); 
 define('PREDICATE_SAME_AS',         'sameAs'); 
 define('PREDICATE_COMMENT',         'comment');
 define('PREDICATE_TYPE',            'type');
 define('PREDICATE_SUBPROPERTY_OF',  'subPropertyOf');
 define('PREDICATE_SUBCLASS_OF',     'subClassOf');
 define('PREDICATE_NAME',            'identifier');
 define('PREDICATE_VALUE',           'value');

 define('PREDICATE_DOMAIN',          'domain');
 define('PREDICATE_RANGE',           'range');

 define('PREDICATE_CAN_HAVE_PROPERTY','canHaveProperty'); // invented inverse of domain
 define('PREDICATE_CAN_USE_ONTOLOGY','canUseOntology'); // invented superset of canHaveProperty
 
 define('PREDICATE_INVERSE_OF',      'inverseOf');

 define('PREDICATE_IS_PART_OF',      'isPartOf');
 define('PREDICATE_DRUPAL_EQUIV',    'drupalEquivalent');


 define('PREDICATE_MODIFIED',        'modified' ); 
 define('PREDICATE_DESCRIPTION',     'description' ); 
 define('PREDICATE_CREATOR',         'creator' ); 
 define('PREDICATE_SORT_ORDER',      'sortOrder' ); 

// Ontologies are collections of terms, independant
// of what namespace the term was defined in 
// (although they do usually go together)
// predicate ontologies are analogous to term taxonomies.
// A term can be in as many ontologies as it likes.

 define('ONTOLOGY_RDF_AXIOMS', 'rdfAxioms'); 
 // these terms should be hidden by default - they are a bit techie

 define('ONTOLOGY_OWL', 'owlOntology'); 
 // these terms are extremely techie, they are used internally by the inference engine
 // By default, terms in this group will be hidden from the admin screen
 
 define('ONTOLOGY_RDF', 'rdfOntology'); 
 // these terms may be available for users

 define('ONTOLOGY_DRUPAL', 'drupalOntology');

 // an empty placeholder for users to put their own terms
 define('ONTOLOGY_LOCAL', 'localOntology');

// If a relationship is derived programatically by a lookup function
// (which means it can always be re-calculated later)
// set its 'depth' to this.
// This implies the storage should not remember this as it may mysteriously change.
 define('RELATIONSHIP_IS_CALCULATED',      -1);

// If a relationship is deduced logically
// (which usually means figuring it out again may be a lot of work)
// set its 'depth' to this.
// This implies the storage may cache it, but it can be discarded in favour of explicit statements.
 define('RELATIONSHIP_IS_DEDUCED',         2);





