<?php

/**
 * Impliment logical characteristic lookups
 */
 
 /**
 * Lookup statements that imply the given query via the inverse rule.
 *
 * Predicate may be blank, which would mean anything that says anything about the subject.
 * @return an array of statements, may be empty, but always an array.
 */
function relationship_query_inverseOf($q){
  if(!$q){debug('no query? this is a waste of time',2);return;}
  
  $statements = array();
  $query = relationship_inverse_statement($q);

  if($query){
    $ds = _relationship_datastore();
    $inverse_statements = $ds->query($query);

    if($inverse_statements){
      debug('Found inverse statements: '.join(', ',array_keys($inverse_statements)),3);
      foreach($inverse_statements as $i => $statement){
        if($s = relationship_inverse_statement($statement)){
          //
          // oddness. What if we find more than one 'inverse'?
          // it's causing pain, but what happens if I support it?
          // what will the logical implications be?
          if(gettype($statement['predicate']) == 'array'){
            foreach($statement['predicate'] as $i => $p){
              $alt_s = $statement; $alt_s['predicate']=$p;
              $statements[_relationship_hash_key($alt_s)] = $alt_s;
            }
          }
          else
          $statements[_relationship_hash_key($s)] = $s;
        }
      }
    } else {
      debug("No results from inverse query "._relationship_hash_key( $query ),4); 
    }
  }
  else {
    debug("There is no inverse to the query. ". _relationship_hash_key( $q ) ,4);
  }
  return $statements;
}


/**
 * Reverse the operands and inverse the predicate for a statement.
 *
 * Eg turn A IsChildOf B into B HasChild A If you already know the reciprocal,
 * pass it in, otherwise it will be retrieved.
 *
 * Helper for relationship_query_inverseOf()
 *
 * Inverse of a statement with a blank predicate returns as an inverse
 * statement with a blank predicate. This is (almost) meaningless as a 
 * statement, but useful as a query.
 * The inverse of the question
 * {this, ?, ?}  - tell me everything about this
 * is
 * {?, ?, this}  - tell me everything that MENTIONS this
 *
 * @param $statement @see statements
 * @param $reciprocal predicate term definition @see predicates
 * @return modified statement,  also changes it by reference. 
 *         Returns null if no inverse predicate exists
 */
function relationship_inverse_statement(&$statement, $reciprocal='') {
  if (! $reciprocal && $statement['predicate']) {
    $term_def = relationship_get_term($statement['predicate']);
    if(!$term_def){ return; } // no such relationship
    $reciprocal = $term_def->inverseOf;
  }
  if(gettype($reciprocal) == 'array'){
    trigger_error("Reciprocal of ".$statement['predicate']." is apparently ".print_r($reciprocal,1)." - this cannot be, there can be only one inverse. This should probably be repaired.");
  }  
  if ($reciprocal || (!$statement['predicate']) ) {
    $recip_statement = array(
      'subject'   => $statement['object'],
      'predicate' => $reciprocal,
      'object'    => $statement['subject'],  
      'inverseOf' => $statement['predicate'],  
      'inverseTitle' => $statement['title'],  
      'title'     => '',
      'depth'     => $statement['depth'] +1,
      'why'       => $statement['predicate']?"Inverse of ".($statement['subject'])." ".($statement['predicate'])." statement ".$statement['rsid']:'wildcard lookup'
   );
  }
  return $statement = $recip_statement;
}

