/** @mainpage Dans RDF Relationship Packages
 *
 * 
 * The Relationships API - where I try to logically say 
 * all things that can be said about a Drupal Node ... in terms of RDF.
 * 
 * - Components of The Relationships package
 *   - @link relationship_api API - The interface to set and get node relationships and metadata  @endlink
 *   - @link ontologies Ontologies - Pluggable sets of terms and types of relationships @endlink 
 *   - @link docs Docs - Theory and background documentation @endlink 
 */ 


/**
 * @defgroup docs Theory and background documentation
 *
 * Dear coder
 *
 * If you are looking at this code, here's a few explanations
 *
 * The function names are long - I tend towards trying to make code make
 * sense when read out loud.
 * Same goes for the variable names.
 *
 * This package looks big - Actually 2/3 (or more) is comments. There's a lot
 * of concepts I wrote about as I went, re-writing or clarifying as the code
 * developed rather than in a separate documentation file. I expected large
 * amounts of this to be pulled out by a doc-gen thing, but doxygen is not
 * co-operating, and I haven't been able to test the output very well. As 
 * such there may be a few inconsistancies with my @ tags.
 *
 * In many cases I use three statements where one line would do. Usually that
 * is to explicitly name an intermediate variable to help with conceptual 
 * understanding - for me. Or to dump debug output in the meantime.
 * But some things may be ready to be made simpler again.
 * The ternary op is cool to write, but always hard to read.
 *
 * I've got lots of (transient) caching going on. It may be excessive, and 
 * could even cause some head-scratching. 
 * It does I guess increase memory requirements a bit - it's a trade-off.
 * Even with logical recursion through statements going on, I'm hoping to be
 * dealing with dozens, not thousands of triples 
 * at a time. ... to be investigated.
 *
 * Turning up the DEBUGLEVEL below will dump lots of info.
 * Comments marked out with # are serious scaffolding - expected to be 
 * removed when I'm happy with the rest of it.
 *
 * Any suggestions on fixing up the coding structure and making the API
 * doc reader work better are welcome.
 *
 * .dan.
 */
/**
 * @{
 */

/** 
 * @defgroup drupal_hooks Drupal Hooks
 * 
 * Nothing exciting to say about these functions.
 *
 * @link http://drupaldocs.org/api/4.6/group/hooks See the drupal Docs for more detail @endlink
 */ 

/** 
 * @defgroup relationship_api API
 * 
 * User/Developer guide to the public methods available in the package.
 *
 * All non-private functions are available to use as utilities, but the functions 
 * listed here as API are explicitly the face of this package, here for other
 * modules to use.
 */ 



/** 
 * @defgroup predicates Predicates
 *
 * About how predicate terms are managed internally.
 *
 * All predicates may optionally have characteristics. It's the 
 * characteristics which give each predicate term its logical structure. 
 * Internally, predicate definitions are retrieved using 
 * relationship_get_term() and are handed around as small objects.
 * Characteristics are stored as an indexed array inside the term
 * object.
 *
 * A sample term may look like:
 *     $parent_def = stdClass(
 *      'name' => 'parentOf',
 *      'uri' => 'http://purl.org/vocab/relationship/parentOf',
 *      'label' => 'Parent Of',
 *      'definition' => 'A person who has given birth to or nurtured and raised this person" ',
 *      'weight' => '2',
 *      'inverseOf' => 'childOf'
 *     )   
 * 
 * Internally, a term is managed as a collection of statements, existing only
 * through the confluence of right sort of information. There is no table and
 * there is no row that defines a term.
 * The statements 
 * 
 * 'nav:parentOf' 'rdf:isType'    'rdfs:Property'
 * 'nav:parentOf' 'rdf:label'     'Parent Of'
 * 'nav:parentOf' 'owl:inverseOf' 'nav:childOf'
 * 
 * ... and more ...
 * come together to tell us what we need to know about the 'parentOf' term.
 *
 * A predicate has a URI as a unique identifier, but for convenience we use
 * only its shortname most of the time. 
 * A statement:
 * 'http://purl.org/vocab/relationship/parentOf' : 'identifier' : 'parentOf'
 * records this. (the identifier of {long URL} is {short name})
 *
 * @see administer_predicates.inc
 *
 * @todo 
 *  My 'characteristics' based on my reading of W3C OWL should actually
 *  be broken down into owl:restriction and  owl:objectProperty ?
 *
 * @todo 
 *  I need to see how and if to use namespaces internally.
 *  Use the RDF isDefinedBy or what?
 *
 * @ingroup docs 
 *
 */

 
/**
 * @defgroup extensions Extending the module
 *
 * Modules may supply their own terms, their own logic
 * and even their own ontology characteristics.
 *
 * An Ontology is a set of meanings. 
 * Defining an ontology plugin module means defining 
 * some terms and what they actually do.
 *
 * By declaring a set of custom predicates, new terms
 * can be applied to node relationships.
 * By giving the predicates characteristics, new logical
 * behaviours can support the terms.
 * By using the usual nodeapi or blocks, the new relationships
 * can be used to change the way pages are presented.
 * @see ontology_navigation.module
 * @see ontology_revision_control.module
 *
 * @ingroup docs Theory and background documentation
 */


/** 
 * @defgroup ontologies Ontologies
 *
 * Groups of logical predicate terms.
 *
 * An overview of Ontologies, as implimented by the relationship.module.
 *
 * Extending the module, defining your own predicates
 * ontologies and behaviours.
 *
 * Modules may supply their own terms,  their own logic
 * and even their own ontology characteristics.
 *
 * An Ontology is a set of meanings. 
 * Defining an ontology plugin module means defining 
 * some terms and what they actually do.
 *
 * By declaring a set of custom predicates, new terms
 * can be applied to node relationship.
 * By giving the predicates characteristics, new logical
 * behaviours can support the terms.
 * A set of relationship definitions could be used to
 * create a new way or threading conversations in discussion groups
 * by defining 'IsReplyTo' and 'IsInThread' then extending it to
 * 'IsRebuttalOf' and 'IsDigression' to see what happens.
 * <http://www.eyrie.org/~zednenem/2002/web-threads/>
 *
 * By using the usual nodeapi or blocks,  the new relationship
 * can be used to change the way pages are presented. For example
 * by providing more than one flavour of 'Next' button.
 * Next in thread,  next in time,  etc
 *
 * @see ontology_rdf.module to import a whole ontology from public RDF schemas.
 * @see ontology_forum.module for an experiment in using relationships to represent a forum.
 * @see ontology_navigation.module as an example
 * @see ontology_geneology.module as an example of making your own schema.
 * @ingroup docs 
 */


/**
 * Return a list of predicate term definitions.
 *
 * Prototype function - modules can declare their own version
 * to extend the predicate vocabulary. Sample code here is only
 * for demonstration!
 *
 * These terms are almost raw database inserts,  and
 * should get inserted into the database when the 
 * module is first installed.
 * 
 * This function should return an array of term definitions.
 * A term definition is an assoc array defining the following key values:
 *
 *   - name     : a shortname describing the predicate
 *   - uri      : a unique identifier to distinguish this use of the word from any other,  anywhere.
 *   - label    : a human-readable version of the term name,  preferably making a joining sentence fragment.
 *   - comment  : a long description,  describing the predicates purpose and behaviour. Irregular caps convention is due to Dublin Core.
 *   - weight   : used for ordering the term in lists. Negative Weighting will HIDE a predicate
 *                from user editing menus. Used for properties that are only ever implied (eg nav:last).
 *   - other characteristics : special parameters that affect the behaviout of the predicate in logical lookups
 *                See apply_characteristics.inc for descriptions of the available characteristics.
 *
 * Your module can also define functions to specifically be triggered 
 * when one of 'your' terms is requested.
 * @ingroup ontologies
 * @ingroup docs  
 * @see relationship_lookup_hook()
 */
function hook_get_terms() {
  
  // Demo Code
  // Not expected to be executed
  
  $list = array(
    'isPartOf' => array (
      'name'       => 'isPartOf',
      'uri'        => 'http://sw.nokia.com/VOC-1/isPartOf',
      'label'      => 'Is In The Vocabulary',
      'definition' => 'A vocabulary to which a term or subvocabulary belongs. " ',
      'type'       => CLASS_PROPERTY,
      'domain'     => CLASS_PROPERTY,
      'range'      => CLASS_ONTOLOGY,
      'inverseOf'  => 'http://www.w3.org/2000/01/rdf-schema#member', 
 
   ),

    'Ontology' => array (
      'name'       => 'Ontology',
      'uri'        => 'http://www.w3.org/2002/07/owl#Ontology',
      'label'      => 'an Ontology',
      'definition' => 'Represents an ontology object class" ',
      'type'       => CLASS_CLASS,
   ),


    'relatedTo' => array (
      'name'       => 'relatedTo',
      'uri'        => 'http://coders.co.nz/projects/xml/navigation#relatedTo',
      'label'      => 'Is Related To',
      'definition' => 'A nonsense definition, The referenced resource is in some way pertinant to the current one.',
      'weight'     => '2',
      'type'       => CLASS_PROPERTY,
   ),
 );

}

/**
 * Executes a custom lookup for a predicate query.
 *
 * Prototype function - modules can declare their own version
 * to extend the predicate vocabulary. Sample code here is only
 * for demonstration!
 *
 * When a module defines a new predicate term, it may also define the 
 * way in which a query on that predicate is resolved.
 * This can either be done by assigning the term some 'characteristics'
 * and letting an inference engine resolve the result, or by implimenting
 * this relationship_lookup_hook() and calculating the query itself.
 *
 * In simple cases, this hook need not be implimented at all.
 * Basic cases (like reciprocal lookups) are best handled using characteristics.
 * But if you need to combine iteration with recursion (like listing all
 * second cousins) the logic can be custom-coded here.
 *
 * Pseudo-parameter predicate_label replaces the word 'hook' in the function name,
 * eg relationship_lookup_hasCousin('Enoch')
 *
 * This function sttempts to return an array containing the statement(s)
 * claiming that something has the stated relationship to the current item.
 * A relationship_lookup_hook() will always return an ARRAY of full STATEMENTS.
 *
 * @param $subject the subject identifier the query is run on. May be null.
 * @param $max_recursion_level
 *
 * @returns an array of statements answering the query: { ?subject ?predicate_name ?}
 * @see relationship_query()
 * @ingroup docs 
 * @ingroup ontologies
 */
function relationship_lookup_hook($subject,  $max_recursion_level=1,  $depth=0) {

  // Demo Code
  // This is just illustrating how you MAY wish
  // to retrieve all 'cousins' of a subject, assuming
  // the hasUncle and ParentOf predicated are also 
  // declared somewhere.

  $statements = array(); 
  
  $uncles = relationship_get_properties($subject, 'hasUncle', $max_recursion_level);
  
  foreach($uncles as $uncle){
    $uncles_children = relationship_get_properties($uncle, 'ParentOf', $max_recursion_level);
    
    foreach($uncles_children as $cousin){
      $statements[] = relationship_statement($subject,  'hasCousin',  $cousin);
    }
  }
  
  // get_titles validates and fleshes-out the statements
  relationship_get_titles($statements);

  return $statements;
  // Always return an array. It could be empty, but that's an informative response.
}


/**
 * @}
 */
